from dsb_spider import log
from dsb_spider.log import ex, getLogger
from dsb_spider.sched import TaskListener
from dsb_spider.tasker import BaseTask, TaskState, TaskSingleton, ImageTaskSingleton, ActivityTaskSingleton

__all__ = ('log', 'ex', 'getLogger',
           'TaskListener'
           'BaseTask', 'TaskState', 'TaskSingleton', 'ImageTaskSingleton', 'ActivityTaskSingleton')



import queue
import traceback
import threading
from dsb_spider import log

logger = log.getLogger('dsb')


class TaskListener():
    def __init__(self, queue, concurrent=3):
        self.queue = queue
        self.concurrent = concurrent
    
    def enqueue(self, record):
        self.queue.put_nowait(record)
    
    def dequeue(self, block):
        task = self.queue.get(block)
        return task
    
    def work(self):
        q = self.queue
        has_task_done = hasattr(q, 'task_done')
        while True:
            try:
                task = self.dequeue(True)
            except queue.Empty:
                break
            
            try:
                if not task:
                    break
                self.handle(task)
                
                if has_task_done:
                    q.task_done()
            except Exception as ex:
                traceback.print_exc()
                logger.errorx(ex)
            finally:
                task.finish()
    
    def handle(self, task):
        logger.debug(f'start: {task}')
        task.do_task()
    
    def start(self):
        for i in range(self.concurrent):
            t = threading.Thread(target=self.work)
            t.daemon = True
            t.start()


def str_to_dict(str1, sep=':'):
    dict1 = {}
    for line in str1.splitlines(True):
        if line.strip():
            p = line.find(sep)
            # print '"'+line[:p]+'":"'+line[p+1:]+'",'
            # print '"'+line[:p]+'":"'+line[p+1:-1]+'",\n',
            dict1[line[:p].strip()] = line[p + 1:-1].strip()
    # print_json(dict1)
    return dict1

def typed_property(name, expected_type, default_value=None):
    storage_name = '_' + name

    @property
    def prop(self):
        return getattr(self, storage_name, default_value)

    @prop.setter
    def prop(self, value):
        if isinstance(expected_type, list):
            if name not in expected_type:
                raise TypeError('{} must be in {}'.format(name, ','.join(expected_type)))
        if not isinstance(value, expected_type):
            raise TypeError('{} must be a {}'.format(name, expected_type))
        setattr(self, storage_name, value)

    @prop.deleter
    def first_name(self):
        raise AttributeError("Can't delete attribute")

    return prop